@@ -43,7 +43,7 @@
 #include "config.h"
 
 #include "bidi.hh"
-#include "debug.h"
+#include "debug.hh"
 #include "vtedefines.hh"
 #include "vteinternal.hh"
 
@@ -96,30 +96,6 @@ BidiRow::set_width(vte::grid::column_t width)
         m_width = width;
 }
 
-/* Converts from logical to visual column. Offscreen columns are mirrored
- * for RTL lines, e.g. (assuming 80 columns) -1 <=> 80, -2 <=> 81 etc. */
-vte::grid::column_t
-BidiRow::log2vis(vte::grid::column_t col) const
-{
-        if (col >= 0 && col < m_width) {
-                return m_log2vis[col];
-        } else {
-                return m_base_rtl ? m_width - 1 - col : col;
-        }
-}
-
-/* Converts from visual to logical column. Offscreen columns are mirrored
- * for RTL lines, e.g. (assuming 80 columns) -1 <=> 80, -2 <=> 81 etc. */
-vte::grid::column_t
-BidiRow::vis2log(vte::grid::column_t col) const
-{
-        if (col >= 0 && col < m_width) {
-                return m_vis2log[col];
-        } else {
-                return m_base_rtl ? m_width - 1 - col : col;
-        }
-}
-
 /* Whether the cell at the given visual position has RTL directionality.
  * For offscreen columns the line's base direction is returned. */
 bool
@@ -726,7 +702,7 @@ BidiRunner::implicit_paragraph(vte::grid::row_t start, vte::grid::row_t end, boo
 
                 /* From vis2log create the log2vis mapping too.
                  * In debug mode assert that we have a bijective mapping. */
-                if (_vte_debug_on (VTE_DEBUG_BIDI)) {
+                _VTE_DEBUG_IF(vte::debug::category::BIDI) {
                         for (tl = 0; tl < width; tl++) {
                                 bidirow->m_log2vis[tl] = -1;
                         }
@@ -736,7 +712,7 @@ BidiRunner::implicit_paragraph(vte::grid::row_t start, vte::grid::row_t end, boo
                         bidirow->m_log2vis[bidirow->m_vis2log[tv]] = tv;
                 }
 
-                if (_vte_debug_on (VTE_DEBUG_BIDI)) {
+                _VTE_DEBUG_IF(vte::debug::category::BIDI) {
                         for (tl = 0; tl < width; tl++) {
                                 vte_assert_cmpint (bidirow->m_log2vis[tl], !=, -1);
                         }
