@@ -19,26 +19,12 @@
 
 #include "config.h"
 
-#include "debug.h"
+#include "debug.hh"
 #include "ring.hh"
 #include "vterowdata.hh"
 
 #include <string.h>
 
-#if WITH_SIXEL
-
-#include "cxx-utils.hh"
-
-/* We should be able to hold a single fullscreen 4K image at most.
- * 35MiB equals 3840 * 2160 * 4 plus a little extra. */
-#define IMAGE_FAST_MEMORY_USED_MAX (35 * 1024 * 1024)
-
-/* Hard limit on number of images to keep around. This limits the impact
- * of potential issues related to algorithmic complexity. */
-#define IMAGE_FAST_COUNT_MAX 4096
-
-#endif /* WITH_SIXEL */
-
 /*
  * Copy the common attributes from VteCellAttr to VteStreamCellAttr or vice versa.
  */
@@ -58,10 +44,10 @@ using namespace vte::base;
 void
 Ring::validate() const
 {
-	_vte_debug_print(VTE_DEBUG_RING,
-			" Delta = %lu, Length = %lu, Next = %lu, Max = %lu, Writable = %lu.\n",
-			m_start, m_end - m_start, m_end,
-			m_max, m_end - m_writable);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Delta = {}, Length = {}, Next = {}, Max = {}, Writable = {}",
+                         m_start, m_end - m_start, m_end,
+                         m_max, m_end - m_writable);
 
 	vte_assert_cmpuint(m_start, <=, m_writable);
 	vte_assert_cmpuint(m_writable, <=, m_end);
@@ -79,7 +65,7 @@ Ring::Ring(row_t max_rows,
           m_has_streams{has_streams},
           m_last_attr{basic_cell.attr}
 {
-	_vte_debug_print(VTE_DEBUG_RING, "New ring %p.\n", this);
+	_vte_debug_print(vte::debug::category::RING, "New ring {}", (void*)this);
 
 	m_array = (VteRowData* ) g_malloc0 (sizeof (m_array[0]) * (m_mask + 1));
 
@@ -138,15 +124,15 @@ Ring::hyperlink_gc()
         VteRowData* row;
         char *used;
 
-        _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                          "hyperlink: GC starting (highest used idx is %d)\n",
-                          m_hyperlink_highest_used_idx);
+        _vte_debug_print(vte::debug::category::HYPERLINK,
+                         "hyperlink: GC starting (highest used idx is {})",
+                         m_hyperlink_highest_used_idx);
 
         m_hyperlink_maybe_gc_counter = 0;
 
         if (m_hyperlink_highest_used_idx == 0) {
-                _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                                  "hyperlink: GC done (no links at all, nothing to do)\n");
+                _vte_debug_print(vte::debug::category::HYPERLINK,
+                                 "hyperlink: GC done (no links at all, nothing to do)");
                 return;
         }
 
@@ -168,9 +154,10 @@ Ring::hyperlink_gc()
 
         for (idx = 1; idx <= m_hyperlink_highest_used_idx; idx++) {
                 if (!GET_BIT(used, idx) && hyperlink_get(idx)->len != 0) {
-                        _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                                          "hyperlink: GC purging link %d to id;uri=\"%s\"\n",
-                                          idx, hyperlink_get(idx)->str);
+                        _vte_debug_print(vte::debug::category::HYPERLINK,
+                                         "hyperlink: GC purging link {} to id;uri=\"{}\"",
+                                         idx,
+                                         hyperlink_get(idx)->str);
                         /* Wipe out the ID and URI itself so it doesn't linger on in the memory for a long time */
                         memset(hyperlink_get(idx)->str, 0, hyperlink_get(idx)->len);
                         g_string_truncate (hyperlink_get(idx), 0);
@@ -181,9 +168,9 @@ Ring::hyperlink_gc()
                m_hyperlink_highest_used_idx--;
         }
 
-        _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                          "hyperlink: GC done (highest used idx is now %d)\n",
-                          m_hyperlink_highest_used_idx);
+        _vte_debug_print(vte::debug::category::HYPERLINK,
+                         "hyperlink: GC done (highest used idx is now {})",
+                         m_hyperlink_highest_used_idx);
 
         g_free (used);
 }
@@ -196,123 +183,14 @@ Ring::hyperlink_maybe_gc(row_t increment)
 {
         m_hyperlink_maybe_gc_counter += increment;
 
-        _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                          "hyperlink: maybe GC, counter at %ld\n",
-                          m_hyperlink_maybe_gc_counter);
+        _vte_debug_print(vte::debug::category::HYPERLINK,
+                         "hyperlink: maybe GC, counter at {}",
+                         m_hyperlink_maybe_gc_counter);
 
         if (m_hyperlink_maybe_gc_counter >= 65536)
                 hyperlink_gc();
 }
 
-#if WITH_SIXEL
-
-void
-Ring::image_gc_region() noexcept
-{
-        cairo_region_t *region = cairo_region_create();
-
-        for (auto rit = m_image_map.rbegin();
-             rit != m_image_map.rend();
-             ) {
-                auto const& image = rit->second;
-                auto const rect = cairo_rectangle_int_t{image->get_left(),
-                                                        image->get_top(),
-                                                        image->get_width(),
-                                                        image->get_height()};
-
-                if (cairo_region_contains_rectangle(region, &rect) == CAIRO_REGION_OVERLAP_IN) {
-                        /* vte::image::Image has been completely overdrawn; delete it */
-
-                        m_image_fast_memory_used -= image->resource_size();
-
-                        /* Apparently this is the cleanest way to erase() with a reverse iterator... */
-                        /* Unlink the image from m_image_by_top_map, then erase it from m_image_map */
-                        unlink_image_from_top_map(image.get());
-                        rit = image_map_type::reverse_iterator{m_image_map.erase(std::next(rit).base())};
-                        continue;
-                }
-
-                cairo_region_union_rectangle(region, &rect);
-                ++rit;
-        }
-
-        cairo_region_destroy(region);
-}
-
-void
-Ring::image_gc() noexcept
-{
-        while (m_image_fast_memory_used > IMAGE_FAST_MEMORY_USED_MAX ||
-               m_image_map.size() > IMAGE_FAST_COUNT_MAX) {
-                if (m_image_map.empty()) {
-                        /* If this happens, we've miscounted somehow. */
-                        break;
-                }
-
-                auto& image = m_image_map.begin()->second;
-                m_image_fast_memory_used -= image->resource_size();
-                unlink_image_from_top_map(image.get());
-                m_image_map.erase(m_image_map.begin());
-        }
-}
-
-void
-Ring::unlink_image_from_top_map(vte::image::Image const* image) noexcept
-{
-        auto [begin, end] = m_image_by_top_map.equal_range(image->get_top());
-
-        for (auto it = begin; it != end; ++it) {
-                if (it->second != image)
-                        continue;
-
-                m_image_by_top_map.erase(it);
-                break;
-        }
-}
-
-void
-Ring::rebuild_image_top_map() /* throws */
-{
-        m_image_by_top_map.clear();
-
-        for (auto it = m_image_map.begin(), end = m_image_map.end();
-             it != end;
-             ++it) {
-                auto const& image = it->second;
-                m_image_by_top_map.emplace(std::piecewise_construct,
-                                           std::forward_as_tuple(image->get_top()),
-                                           std::forward_as_tuple(image.get()));
-        }
-}
-
-bool
-Ring::rewrap_images_in_range(Ring::image_by_top_map_type::iterator& it,
-                             size_t text_start_ofs,
-                             size_t text_end_ofs,
-                             row_t new_row_index) noexcept
-{
-        for (auto const end = m_image_by_top_map.end();
-             it != end;
-             ++it) {
-                auto const& image = it->second;
-                auto ofs = CellTextOffset{};
-
-                if (!frozen_row_column_to_text_offset(image->get_top(), 0, &ofs))
-                        return false;
-
-                if (ofs.text_offset >= text_end_ofs)
-                        break;
-
-                if (ofs.text_offset >= text_start_ofs && ofs.text_offset < text_end_ofs) {
-                        image->set_top(new_row_index);
-                }
-        }
-
-        return true;
-}
-
-#endif /* WITH_SIXEL */
-
 /*
  * Find existing idx for the hyperlink or allocate a new one.
  *
@@ -338,9 +216,9 @@ Ring::get_hyperlink_idx_no_update_current(char const* hyperlink)
         auto const last_idx = m_hyperlink_highest_used_idx + 1;
         for (idx = 1; idx < last_idx; ++idx) {
                 if (strcmp(hyperlink_get(idx)->str, hyperlink) == 0) {
-                        _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                                          "get_hyperlink_idx: already existing idx %d for id;uri=\"%s\"\n",
-                                          idx, hyperlink);
+                        _vte_debug_print(vte::debug::category::HYPERLINK,
+                                         "get_hyperlink_idx: already existing idx {} for id;uri=\"{}\"",
+                                         idx, hyperlink);
                         return idx;
                 }
         }
@@ -351,9 +229,9 @@ Ring::get_hyperlink_idx_no_update_current(char const* hyperlink)
         /* Another linear search for an empty slot where a GString is already allocated */
         for (idx = 1; idx < m_hyperlinks->len; idx++) {
                 if (hyperlink_get(idx)->len == 0) {
-                        _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                                          "get_hyperlink_idx: reassigning old idx %d for id;uri=\"%s\"\n",
-                                          idx, hyperlink);
+                        _vte_debug_print(vte::debug::category::HYPERLINK,
+                                         "get_hyperlink_idx: reassigning old idx {} for id;uri=\"{}\"",
+                                         idx, hyperlink);
                         /* Grow size if required, however, never shrink to avoid long-term memory fragmentation. */
                         g_string_append_len (hyperlink_get(idx), hyperlink, len);
                         m_hyperlink_highest_used_idx = MAX (m_hyperlink_highest_used_idx, idx);
@@ -367,16 +245,16 @@ Ring::get_hyperlink_idx_no_update_current(char const* hyperlink)
         /* VTE_HYPERLINK_COUNT_MAX should be big enough for this not to happen under
            normal circumstances. Anyway, it's cheap to protect against extreme ones. */
         if (m_hyperlink_highest_used_idx == VTE_HYPERLINK_COUNT_MAX) {
-                _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                                  "get_hyperlink_idx: idx 0 (ran out of available idxs) for id;uri=\"%s\"\n",
-                                  hyperlink);
+                _vte_debug_print(vte::debug::category::HYPERLINK,
+                                 "get_hyperlink_idx: idx 0 (ran out of available idxs) for id;uri=\"{}\"",
+                                 hyperlink);
                 return 0;
         }
 
         idx = ++m_hyperlink_highest_used_idx;
-        _vte_debug_print (VTE_DEBUG_HYPERLINK,
-                          "get_hyperlink_idx: brand new idx %d for id;uri=\"%s\"\n",
-                          idx, hyperlink);
+        _vte_debug_print(vte::debug::category::HYPERLINK,
+                         "get_hyperlink_idx: brand new idx {} for id;uri=\"{}\"",
+                         idx, hyperlink);
         str = g_string_new_len (hyperlink, len);
         g_ptr_array_add(m_hyperlinks, str);
 
@@ -416,7 +294,9 @@ Ring::freeze_row(row_t position,
 	int i;
         gboolean froze_hyperlink = FALSE;
 
-	_vte_debug_print (VTE_DEBUG_RING, "Freezing row %lu.\n", position);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Freezing row {}",
+                         position);
 
         g_assert(m_has_streams);
 
@@ -533,7 +413,9 @@ Ring::thaw_row(row_t position,
                 *hyperlink = m_hyperlink_buf;
         }
 
-	_vte_debug_print (VTE_DEBUG_RING, "Thawing row %lu.\n", position);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Thawing row {}",
+                         position);
 
         g_assert(m_has_streams);
 
@@ -598,7 +480,7 @@ Ring::thaw_row(row_t position,
 		}
 
 		cell.attr = attr;
-                _VTE_DEBUG_IF(VTE_DEBUG_RING | VTE_DEBUG_HYPERLINK) {
+                _VTE_DEBUG_IF(vte::debug::category::RING | vte::debug::category::HYPERLINK) {
                         /* Debug: Reverse the colors for the stream's contents. */
                         if (!do_truncate) {
                                 cell.attr.attr ^= VTE_ATTR_REVERSE;
@@ -647,7 +529,7 @@ Ring::thaw_row(row_t position,
            which is the reason for the hyperlink's length being repeated after the hyperlink itself. */
 	if (do_truncate) {
 		gsize attr_stream_truncate_at = records[0].attr_start_offset;
-		_vte_debug_print (VTE_DEBUG_RING, "Truncating\n");
+		_vte_debug_print(vte::debug::category::RING, "Truncating");
 		if (records[0].text_start_offset <= m_last_attr_text_start_offset) {
 			/* Check the previous attr record. If its text ends where truncating, this attr record also needs to be removed. */
                         guint16 hyperlink_length;
@@ -655,7 +537,7 @@ Ring::thaw_row(row_t position,
                                 vte_assert_cmpuint (hyperlink_length, <=, VTE_HYPERLINK_TOTAL_LENGTH_MAX);
                                 if (_vte_stream_read (m_attr_stream, attr_stream_truncate_at - 2 - hyperlink_length - sizeof (attr_change), (char *) &attr_change, sizeof (attr_change))) {
                                         if (records[0].text_start_offset == attr_change.text_end_offset) {
-                                                _vte_debug_print (VTE_DEBUG_RING, "... at attribute change\n");
+                                                _vte_debug_print(vte::debug::category::RING, "... at attribute change");
                                                 attr_stream_truncate_at -= sizeof (attr_change) + hyperlink_length + 2;
                                         }
 				}
@@ -693,7 +575,9 @@ Ring::thaw_row(row_t position,
 void
 Ring::reset_streams(row_t position)
 {
-	_vte_debug_print (VTE_DEBUG_RING, "Reseting streams to %lu.\n", position);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Reseting streams to {}",
+                         position);
 
 	if (m_has_streams) {
 		_vte_stream_reset(m_row_stream, position * sizeof(RowRecord));
@@ -708,19 +592,14 @@ Ring::reset_streams(row_t position)
 Ring::row_t
 Ring::reset()
 {
-        _vte_debug_print (VTE_DEBUG_RING, "Reseting the ring at %lu.\n", m_end);
+        _vte_debug_print(vte::debug::category::RING,
+                         "Reseting the ring at {}",
+                         m_end);
 
         reset_streams(m_end);
         m_start = m_writable = m_end;
         m_cached_row_num = (row_t)-1;
 
-#if WITH_SIXEL
-        m_image_by_top_map.clear();
-        m_image_map.clear();
-        m_next_image_priority = 0;
-        m_image_fast_memory_used = 0;
-#endif
-
         return m_end;
 }
 
@@ -731,7 +610,9 @@ Ring::index(row_t position)
 		return get_writable_index(position);
 
 	if (m_cached_row_num != position) {
-		_vte_debug_print(VTE_DEBUG_RING, "Caching row %lu.\n", position);
+		_vte_debug_print(vte::debug::category::RING,
+                                 "Caching row {}",
+                                 position);
                 thaw_row(position, &m_cached_row, false, -1, nullptr);
 		m_cached_row_num = position;
 	}
@@ -957,7 +838,9 @@ Ring::ensure_writable_room()
 		m_mask = (m_mask << 1) + 1;
         } while (m_mask < m_visible_rows + 1 || m_writable + m_mask + 1 <= m_end);
 
-	_vte_debug_print(VTE_DEBUG_RING, "Enlarging writable array from %lu to %lu\n", old_mask, m_mask);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Enlarging writable array from {} to {}",
+                         old_mask, m_mask);
 
 	m_array = (VteRowData* ) g_malloc0(sizeof (m_array[0]) * (m_mask + 1));
 
@@ -980,7 +863,9 @@ Ring::ensure_writable_room()
 void
 Ring::resize(row_t max_rows)
 {
-	_vte_debug_print(VTE_DEBUG_RING, "Resizing to %lu.\n", max_rows);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Resizing to {}",
+                         max_rows);
 
 	validate();
 
@@ -1002,7 +887,9 @@ Ring::shrink(row_t max_len)
 	if (length() <= max_len)
 		return;
 
-	_vte_debug_print(VTE_DEBUG_RING, "Shrinking to %lu.\n", max_len);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Shrinking to {}",
+                         max_len);
 
 	validate();
 
@@ -1035,7 +922,9 @@ Ring::insert(row_t position, guint8 bidi_flags)
 	row_t i;
 	VteRowData* row, tmp;
 
-	_vte_debug_print(VTE_DEBUG_RING, "Inserting at position %lu.\n", position);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Inserting at position {}",
+                         position);
 	validate();
 
 	maybe_discard_one_row();
@@ -1073,7 +962,9 @@ Ring::remove(row_t position)
 	row_t i;
 	VteRowData tmp;
 
-	_vte_debug_print(VTE_DEBUG_RING, "Removing item at position %lu.\n", position);
+	_vte_debug_print(vte::debug::category::RING,
+                         "Removing item at position {}",
+                         position);
         validate();
 
 	if (G_UNLIKELY(!contains(position)))
@@ -1179,16 +1070,7 @@ Ring::frozen_row_column_to_text_offset(row_t position,
 	} else
 		records[1].text_start_offset = _vte_stream_head(m_text_stream);
 
-	offset->fragment_cells = 0;
-	offset->eol_cells = -1;
-	offset->text_offset = records[0].text_start_offset;
-
-        /* Save some work if we're in column 0. This holds true for images, whose column
-         * positions are disregarded for the purposes of wrapping. */
-        if (column == 0)
-                return true;
-
-        g_string_set_size (buffer, records[1].text_start_offset - records[0].text_start_offset);
+	g_string_set_size (buffer, records[1].text_start_offset - records[0].text_start_offset);
 	if (!_vte_stream_read(m_text_stream, records[0].text_start_offset, buffer->str, buffer->len))
 		return false;
 
@@ -1200,6 +1082,8 @@ Ring::frozen_row_column_to_text_offset(row_t position,
 	/* row and buffer now contain the same text, in different representation */
 
 	/* count the number of characters up to the given column */
+	offset->fragment_cells = 0;
+	offset->eol_cells = -1;
 	num_chars = 0;
 	for (i = 0, cell = row->cells; i < row->len && i < column; i++, cell++) {
 		if (G_LIKELY (!cell->attr.fragment())) {
@@ -1220,7 +1104,7 @@ Ring::frozen_row_column_to_text_offset(row_t position,
 		off++;
 		if ((buffer->str[off] & 0xC0) != 0x80) num_chars--;
 	}
-	offset->text_offset += off;
+	offset->text_offset = records[0].text_start_offset + off;
 	return true;
 }
 
@@ -1332,13 +1216,11 @@ Ring::rewrap(column_t columns,
 	gsize paragraph_len;  /* excluding trailing '\n' */
 	gsize attr_offset;
 	gsize old_ring_end;
-#if WITH_SIXEL
-	auto image_it = m_image_by_top_map.begin();
-#endif
 
 	if (G_UNLIKELY(length() == 0))
 		return;
-	_vte_debug_print(VTE_DEBUG_RING, "Ring before rewrapping:\n");
+	_vte_debug_print(vte::debug::category::RING,
+                         "Ring before rewrapping:");
         validate();
 	new_row_stream = _vte_file_stream_new();
 
@@ -1358,10 +1240,14 @@ Ring::rewrap(column_t columns,
 		if (!frozen_row_column_to_text_offset(markers[i]->row, markers[i]->col, &marker_text_offsets[i]))
 			goto err;
 		new_markers[i].row = new_markers[i].col = -1;
-		_vte_debug_print(VTE_DEBUG_RING,
-				"Marker #%d old coords:  row %ld  col %ld  ->  text_offset %" G_GSIZE_FORMAT " fragment_cells %d  eol_cells %d\n",
-				i, markers[i]->row, markers[i]->col, marker_text_offsets[i].text_offset,
-				marker_text_offsets[i].fragment_cells, marker_text_offsets[i].eol_cells);
+		_vte_debug_print(vte::debug::category::RING,
+                                 "Marker #{} old coords:  row {}  col {}  ->  text_offset {} fragment_cells {}  eol_cells {}",
+                                 i,
+                                 markers[i]->row,
+                                 markers[i]->col,
+                                 marker_text_offsets[i].text_offset,
+                                 marker_text_offsets[i].fragment_cells,
+                                 marker_text_offsets[i].eol_cells);
 	}
 
 	/* Prepare for rewrapping */
@@ -1389,8 +1275,8 @@ Ring::rewrap(column_t columns,
 		RowRecord new_record;
 		column_t col = 0;
 
-		_vte_debug_print(VTE_DEBUG_RING,
-				"  Old paragraph:  row %lu  (text_offset %" G_GSIZE_FORMAT ")  up to (exclusive)  ",  /* no '\n' */
+		_vte_debug_print(vte::debug::category::RING,
+				"  Old paragraph:  row {}  (text_offset {})  up to (exclusive)",
                                  old_row_index - 1,
                                  paragraph_start_text_offset);
 		while (old_row_index <= m_end) {
@@ -1412,12 +1298,13 @@ Ring::rewrap(column_t columns,
 		paragraph_len = paragraph_end_text_offset - paragraph_start_text_offset;
 		if (!prev_record_was_soft_wrapped)  /* The last paragraph can be soft wrapped! */
 			paragraph_len--;  /* Strip trailing '\n' */
-		_vte_debug_print(VTE_DEBUG_RING,
-				"row %lu  (text_offset %" G_GSIZE_FORMAT ")%s  len %" G_GSIZE_FORMAT "  is_ascii %d\n",
+		_vte_debug_print(vte::debug::category::RING,
+				"  row {}  (text_offset {}){}  len {}  is_ascii {}",
                                  old_row_index - 1,
                                  paragraph_end_text_offset,
-				prev_record_was_soft_wrapped ? "  soft_wrapped" : "",
-				paragraph_len, paragraph_is_ascii);
+                                 prev_record_was_soft_wrapped ? "  soft_wrapped" : "",
+                                 paragraph_len,
+                                 paragraph_is_ascii);
 		/* Wrap the paragraph */
 		if (attr_change.text_end_offset <= text_offset) {
 			/* Attr change at paragraph boundary, advance to next attr. */
@@ -1474,27 +1361,22 @@ Ring::rewrap(column_t columns,
                                                 new_record.width = col;
 						new_record.soft_wrapped = 1;
 						_vte_stream_append(new_row_stream, (char const* ) &new_record, sizeof (new_record));
-						_vte_debug_print(VTE_DEBUG_RING,
-								"    New row %ld  text_offset %" G_GSIZE_FORMAT "  attr_offset %" G_GSIZE_FORMAT "  soft_wrapped\n",
-								new_row_index,
-								new_record.text_start_offset, new_record.attr_start_offset);
+						_vte_debug_print(vte::debug::category::RING,
+                                                                 "    New row {}  text_offset {}  attr_offset {}  soft_wrapped",
+                                                                 new_row_index,
+                                                                 new_record.text_start_offset,
+                                                                 new_record.attr_start_offset);
 						for (i = 0; i < num_markers; i++) {
 							if (G_UNLIKELY (marker_text_offsets[i].text_offset >= new_record.text_start_offset &&
 									marker_text_offsets[i].text_offset < text_offset)) {
 								new_markers[i].row = new_row_index;
-								_vte_debug_print(VTE_DEBUG_RING,
-										"      Marker #%d will be here in row %lu\n", i, new_row_index);
+								_vte_debug_print(vte::debug::category::RING,
+										"      Marker #{} will be here in row {}",
+                                                                                 i,
+                                                                                 new_row_index);
 							}
 						}
 
-#if WITH_SIXEL
-						if (!rewrap_images_in_range(image_it,
-                                                                            new_record.text_start_offset,
-                                                                            text_offset,
-                                                                            new_row_index))
-							goto err;
-#endif
-
 						new_row_index++;
 						new_record.text_start_offset = text_offset;
 						new_record.attr_start_offset = attr_offset;
@@ -1531,27 +1413,22 @@ Ring::rewrap(column_t columns,
                 new_record.width = col;
 		new_record.soft_wrapped = prev_record_was_soft_wrapped;
 		_vte_stream_append(new_row_stream, (char const* ) &new_record, sizeof (new_record));
-		_vte_debug_print(VTE_DEBUG_RING,
-				"    New row %ld  text_offset %" G_GSIZE_FORMAT "  attr_offset %" G_GSIZE_FORMAT "\n",
-				new_row_index,
-				new_record.text_start_offset, new_record.attr_start_offset);
+		_vte_debug_print(vte::debug::category::RING,
+                                 "    New row {}  text_offset {}  attr_offset {}",
+                                 new_row_index,
+                                 new_record.text_start_offset,
+                                 new_record.attr_start_offset);
 		for (i = 0; i < num_markers; i++) {
 			if (G_UNLIKELY (marker_text_offsets[i].text_offset >= new_record.text_start_offset &&
 					marker_text_offsets[i].text_offset < paragraph_end_text_offset)) {
 				new_markers[i].row = new_row_index;
-				_vte_debug_print(VTE_DEBUG_RING,
-						"      Marker #%d will be here in row %lu\n", i, new_row_index);
+				_vte_debug_print(vte::debug::category::RING,
+                                                 "      Marker #{} will be here in row {}",
+                                                 i,
+                                                 new_row_index);
 			}
 		}
 
-#if WITH_SIXEL
-		if (!rewrap_images_in_range(image_it,
-                                            new_record.text_start_offset,
-                                            paragraph_end_text_offset,
-                                            new_row_index))
-			goto err;
-#endif
-
 		new_row_index++;
 		paragraph_start_text_offset = paragraph_end_text_offset;
 	}
@@ -1577,32 +1454,27 @@ Ring::rewrap(column_t columns,
                          * It would be a bit cumbersome to refactor the code to still revert here. Choose a simple solution. */
                         new_markers[i].col = 0;
                 }
-		_vte_debug_print(VTE_DEBUG_RING,
-				"Marker #%d new coords:  text_offset %" G_GSIZE_FORMAT "  fragment_cells %d  eol_cells %d  ->  row %ld  col %ld\n",
-				i, marker_text_offsets[i].text_offset, marker_text_offsets[i].fragment_cells,
-				marker_text_offsets[i].eol_cells, new_markers[i].row, new_markers[i].col);
+		_vte_debug_print(vte::debug::category::RING,
+                                 "Marker #{} new coords:  text_offset {}  fragment_cells {}  eol_cells {}  ->  row {}  col {}",
+                                 i,
+                                 marker_text_offsets[i].text_offset,
+                                 marker_text_offsets[i].fragment_cells,
+                                 marker_text_offsets[i].eol_cells,
+                                 new_markers[i].row, new_markers[i].col);
 		markers[i]->row = new_markers[i].row;
 		markers[i]->col = new_markers[i].col;
 	}
 	g_free(marker_text_offsets);
 	g_free(new_markers);
 
-#if WITH_SIXEL
-        try {
-                rebuild_image_top_map();
-        } catch (...) {
-                vte::log_exception();
-        }
-#endif
-
-	_vte_debug_print(VTE_DEBUG_RING, "Ring after rewrapping:\n");
+	_vte_debug_print(vte::debug::category::RING, "Ring after rewrapping:");
         validate();
 	return;
 
 err:
 #if VTE_DEBUG
-	_vte_debug_print(VTE_DEBUG_RING,
-			"Error while rewrapping\n");
+	_vte_debug_print(vte::debug::category::RING,
+			"Error while rewrapping");
 	g_assert_not_reached();
 #endif
 	g_object_unref(new_row_stream);
@@ -1655,7 +1527,7 @@ Ring::write_contents(GOutputStream* stream,
 {
 	row_t i;
 
-	_vte_debug_print(VTE_DEBUG_RING, "Writing contents to GOutputStream.\n");
+	_vte_debug_print(vte::debug::category::RING, "Writing contents to GOutputStream");
 
 	if (m_start < m_writable)
 	{
@@ -1698,56 +1570,3 @@ Ring::write_contents(GOutputStream* stream,
 
 	return true;
 }
-
-#if WITH_SIXEL
-
-/**
- * Ring::append_image:
- * @surface: A Cairo surface object
- * @pixelwidth: vte::image::Image width in pixels
- * @pixelheight: vte::image::Image height in pixels
- * @left: Left position of image in cell units
- * @top: Top position of image in cell units
- * @cell_width: Width of image in cell units
- * @cell_height: Height of image in cell units
- *
- * Append an image to the internal image list.
- */
-void
-Ring::append_image(vte::Freeable<cairo_surface_t> surface,
-                   int pixelwidth,
-                   int pixelheight,
-                   long left,
-                   long top,
-                   long cell_width,
-                   long cell_height) /* throws */
-{
-        auto const priority = m_next_image_priority;
-        auto [it, success] = m_image_map.try_emplace
-                (priority, // key
-                 std::make_unique<vte::image::Image>(std::move(surface),
-                                                     priority,
-                                                     pixelwidth,
-                                                     pixelheight,
-                                                     left,
-                                                     top,
-                                                     cell_width,
-                                                     cell_height));
-        if (!success)
-                return;
-
-        ++m_next_image_priority;
-
-        auto const& image = it->second;
-
-        m_image_by_top_map.emplace(std::piecewise_construct,
-                                   std::forward_as_tuple(image->get_top()),
-                                   std::forward_as_tuple(image.get()));
-
-        m_image_fast_memory_used += image->resource_size ();
-
-        image_gc_region();
-        image_gc();
-}
-
-#endif /* WITH_SIXEL */
