@@ -21,7 +21,6 @@
 #include <gtk/gtk.h>
 #include <gtk/gtk-a11y.h>
 #include <string.h>
-#include "debug.h"
 #include <vte/vte.h>
 #include "vteaccess.h"
 #include "vteinternal.hh"
@@ -142,9 +141,6 @@ xy_from_offset (VteTerminalAccessiblePrivate *priv,
 static void
 emit_text_caret_moved(GObject *object, glong caret)
 {
-	_vte_debug_print(VTE_DEBUG_SIGNALS|VTE_DEBUG_ALLY,
-			"Accessibility peer emitting "
-			"`text-caret-moved'.\n");
 	g_signal_emit_by_name(object, "text-caret-moved", caret);
 }
 
@@ -159,12 +155,6 @@ emit_text_changed_insert(GObject *object,
 	/* Convert the byte offsets to character offsets. */
 	start = g_utf8_pointer_to_offset (text, text + offset);
 	count = g_utf8_pointer_to_offset (text + offset, text + offset + len);
-	_vte_debug_print(VTE_DEBUG_SIGNALS|VTE_DEBUG_ALLY,
-			"Accessibility peer emitting "
-			"`text-changed::insert' (%ld, %ld) (%ld, %ld).\n"
-			"Inserted text was `%.*s'.\n",
-			offset, len, start, count,
-			(int) len, text + offset);
 	g_signal_emit_by_name(object, "text-changed::insert", start, count);
 }
 
@@ -179,12 +169,6 @@ emit_text_changed_delete(GObject *object,
 	/* Convert the byte offsets to characters. */
 	start = g_utf8_pointer_to_offset (text, text + offset);
 	count = g_utf8_pointer_to_offset (text + offset, text + offset + len);
-	_vte_debug_print(VTE_DEBUG_SIGNALS|VTE_DEBUG_ALLY,
-			"Accessibility peer emitting "
-			"`text-changed::delete' (%ld, %ld) (%ld, %ld).\n"
-			"Deleted text was `%.*s'.\n",
-			offset, len, start, count,
-			(int) len, text + offset);
 	g_signal_emit_by_name(object, "text-changed::delete", start, count);
 }
 
@@ -302,10 +286,6 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 			 * it's a new line and we need to keep track of where
 			 * it is. */
 			if ((i == 0) || (attrs.row != row)) {
-				_vte_debug_print(VTE_DEBUG_ALLY,
-						"Row %d/%ld begins at %u.\n",
-						priv->snapshot_linebreaks->len,
-						attrs.row, i);
 				g_array_append_val(priv->snapshot_linebreaks, i);
 			}
 			row = attrs.row;
@@ -318,8 +298,6 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 
 	/* Update the caret position. */
 	vte_terminal_get_cursor_position(terminal, &ccol, &crow);
-	_vte_debug_print(VTE_DEBUG_ALLY,
-			"Cursor at (%ld, " "%ld).\n", ccol, crow);
 
 	/* Get the offsets to the beginnings of each line. */
 	caret = 0;
@@ -345,12 +323,6 @@ vte_terminal_accessible_update_private_data_if_needed(VteTerminalAccessible *acc
 
 	/* Done updating the caret position, whether we needed to or not. */
 	priv->snapshot_caret_invalid = FALSE;
-
-	_vte_debug_print(VTE_DEBUG_ALLY,
-			"Refreshed accessibility snapshot, "
-			"%ld cells, %ld characters.\n",
-			(long)vte_char_attr_list_get_size(&priv->snapshot_attributes),
-			(long)priv->snapshot_characters->len);
 }
 
 static void
@@ -386,6 +358,12 @@ _vte_terminal_accessible_text_modified(VteTerminalAccessible* accessible)
 	glong offset, caret_offset, olen, clen;
 	gint old_snapshot_caret;
 
+        auto widget = gtk_accessible_get_widget(GTK_ACCESSIBLE(accessible));
+        auto terminal = VTE_TERMINAL(widget);
+
+        if (!vte_terminal_get_enable_a11y (terminal))
+                return;
+
 	old_snapshot_caret = priv->snapshot_caret;
 	priv->snapshot_contents_invalid = TRUE;
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
@@ -487,6 +465,44 @@ _vte_terminal_accessible_text_modified(VteTerminalAccessible* accessible)
         g_array_free(old_characters, TRUE);
 }
 
+/* Compute a simple diff between the `before` string (before_len bytes) and the
+   `after` string (after_len bytes). We here only check for common head and
+   tail.
+
+   If the strings are equal, this returns FALSE.
+
+   If the strings differ, this returns TRUE, and *head_len is set to the number
+   of bytes that are identical at the beginning of `before` and `after`, and
+   *tail_len is set to the number of bytes that are identical at the end of
+   `before` and `after`. */
+static gboolean
+check_diff(const gchar *before, guint before_len,
+	   const gchar *after, guint after_len,
+	   guint *head_len, guint *tail_len)
+{
+	guint i;
+
+	for (i = 0; i < before_len && i < after_len; i++)
+		if (before[i] != after[i])
+			break;
+
+	if (i == before_len && i == after_len)
+		/* They are identical. */
+		return FALSE;
+
+	/* They started diverging at i. */
+	*head_len = i;
+
+	for (i = 1; i <= before_len - *head_len && i <= after_len - *head_len; i++)
+		if (before[before_len - i] != after[after_len - i])
+			break;
+
+	/* They finished diverging here. */
+	*tail_len = i - 1;
+
+	return TRUE;
+}
+
 void
 _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
                                        long howmuch)
@@ -494,7 +510,7 @@ _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
 	struct _VteCharAttributes attr;
 	long delta, row_count;
-	guint i, len;
+	guint drop, old_len, new_len, old_common, new_common;
 
         /* TODOegmont: Fix this for smooth scrolling */
         /* g_assert(howmuch != 0); */
@@ -503,6 +519,9 @@ _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
         auto widget = gtk_accessible_get_widget(GTK_ACCESSIBLE(accessible));
         auto terminal = VTE_TERMINAL(widget);
 
+        if (!vte_terminal_get_enable_a11y (terminal))
+                return;
+
         row_count = vte_terminal_get_row_count(terminal);
 	if (((howmuch < 0) && (howmuch <= -row_count)) ||
 	    ((howmuch > 0) && (howmuch >= row_count))) {
@@ -531,6 +550,7 @@ _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
                 vte_terminal_accessible_maybe_emit_text_caret_moved(accessible);
 		return;
 	}
+
 	/* Find the start point. */
 	delta = 0;
         if (vte_char_attr_list_get_size(&priv->snapshot_attributes) > 0) {
@@ -540,85 +560,207 @@ _vte_terminal_accessible_text_scrolled(VteTerminalAccessible* accessible,
 	/* We scrolled up, so text was added at the top and removed
 	 * from the bottom. */
 	if ((howmuch < 0) && (howmuch > -row_count)) {
-		gboolean inserted = FALSE;
 		howmuch = -howmuch;
 		if (priv->snapshot_text != NULL) {
+			old_len = vte_char_attr_list_get_size(&priv->snapshot_attributes);
+
 			/* Find the first byte that scrolled off. */
-			for (i = 0; i < vte_char_attr_list_get_size(&priv->snapshot_attributes); i++) {
-                                attr = *vte_char_attr_list_get(&priv->snapshot_attributes, i);
+			for (old_common = 0; old_common < old_len; old_common++) {
+				attr = *vte_char_attr_list_get(&priv->snapshot_attributes, old_common);
 				if (attr.row >= delta + row_count - howmuch) {
 					break;
 				}
 			}
-			if (i < vte_char_attr_list_get_size(&priv->snapshot_attributes)) {
-				/* The rest of the string was deleted -- make a note. */
+			if (old_common < old_len) {
+				drop = old_len - old_common;
+
+				GString *old_text;
+				GArray *old_characters;
+
+				/* Refresh. */
+				priv->snapshot_contents_invalid = TRUE;
+				vte_terminal_accessible_update_private_data_if_needed(accessible,
+										      &old_text,
+										      &old_characters);
+
+				GString *new_text = priv->snapshot_text;
+				GArray *new_characters = priv->snapshot_characters;
+				new_len = new_text->len;
+
+				if (old_common > new_len)
+					/* Not only did we scroll, but we even removed some text */
+					new_common = new_len;
+				else
+					new_common = old_common;
+
+				guint head_len, tail_len;
+				gboolean diff;
+
+				/* Check for more difference than just the addition at top and removal at bottom */
+				diff = check_diff(old_text->str, old_common,
+						  new_text->str + new_len - new_common, new_common,
+						  &head_len, &tail_len);
+
+				/* Note: to keep offsets correct, we need to
+				 * notify deletions from bottom to top and
+				 * additions from top to bottom
+				 */
+
+				/* Temporarily switch to old text to emit deletion events */
+				priv->snapshot_text = old_text;
+				priv->snapshot_characters = old_characters;
+
+				/* Delete bottom first */
 				emit_text_changed_delete(G_OBJECT(accessible),
-						priv->snapshot_text->str,
-						i,
-						vte_char_attr_list_get_size(&priv->snapshot_attributes) - i);
-			}
-			inserted = TRUE;
-		}
-		/* Refresh.  Note that i is now the length of the data which
-		 * we expect to have left over. */
-		priv->snapshot_contents_invalid = TRUE;
-		vte_terminal_accessible_update_private_data_if_needed(accessible,
-								      NULL,
-								      NULL);
-		/* If we now have more text than before, the initial portion
-		 * was added. */
-		if (inserted) {
-			len = priv->snapshot_text->len;
-			if (len > i) {
-				emit_text_changed_insert(G_OBJECT(accessible),
-							 priv->snapshot_text->str,
-							 0,
-							 len - i);
+						old_text->str,
+						old_common,
+						drop);
+
+				if (diff)
+					/* Not only did we scroll, but we also modified some bits */
+					/* Delete middle */
+					emit_text_changed_delete(G_OBJECT(accessible),
+								 old_text->str,
+								 head_len,
+								 old_common - tail_len - head_len);
+
+				/* Switch to new text */
+				priv->snapshot_text = new_text;
+				priv->snapshot_characters = new_characters;
+
+				g_string_free(old_text, TRUE);
+				g_array_free(old_characters, TRUE);
+
+				/* If we now have more text than before, the initial portion
+				 * was added. */
+				if (new_len > new_common)
+					/* Add top */
+					emit_text_changed_insert(G_OBJECT(accessible),
+								 new_text->str,
+								 0,
+								 new_len - new_common);
+
+				if (diff)
+					/* Add middle */
+					emit_text_changed_insert(G_OBJECT(accessible),
+								 new_text->str,
+								 new_len - new_common + head_len,
+								 new_common - tail_len - head_len);
 			}
+		} else {
+			/* Just refresh. */
+			priv->snapshot_contents_invalid = TRUE;
+			vte_terminal_accessible_update_private_data_if_needed(accessible,
+									      NULL,
+									      NULL);
 		}
+
                 vte_terminal_accessible_maybe_emit_text_caret_moved(accessible);
 		return;
 	}
 	/* We scrolled down, so text was added at the bottom and removed
 	 * from the top. */
 	if ((howmuch > 0) && (howmuch < row_count)) {
-		gboolean inserted = FALSE;
 		if (priv->snapshot_text != NULL) {
+			/* Leave trailing '\n' untouched, so that the exposed text always contains a trailing '\n',
+			   insertion happens in front of it: bug 657960 */
+			old_len = vte_char_attr_list_get_size(&priv->snapshot_attributes) - 1;
+
 			/* Find the first byte that wasn't scrolled off the top. */
-			for (i = 0; i < vte_char_attr_list_get_size(&priv->snapshot_attributes); i++) {
-                                attr = *vte_char_attr_list_get(&priv->snapshot_attributes, i);
+			for (drop = 0; drop < old_len; drop++) {
+				attr = *vte_char_attr_list_get(&priv->snapshot_attributes, drop);
 				if (attr.row >= delta + howmuch) {
 					break;
 				}
 			}
-			/* That many bytes disappeared -- make a note. */
+
+			/* Figure out how much text was common, and refresh. */
+			old_common = old_len - drop;
+
+			GString *old_text;
+			GArray *old_characters;
+
+			priv->snapshot_contents_invalid = TRUE;
+			vte_terminal_accessible_update_private_data_if_needed(accessible,
+									      &old_text,
+									      &old_characters);
+
+			GString *new_text = priv->snapshot_text;
+			GArray *new_characters = priv->snapshot_characters;
+			new_len = new_text->len - 1;
+
+			if (old_common > new_len)
+				/* Not only did we scroll, but we even removed some text */
+				new_common = new_len;
+			else
+				new_common = old_common;
+
+			guint head_len, tail_len;
+			guint extra_add = 0;
+			gboolean diff;
+
+			/* Check for more difference than just the removal at top and addition at bottom */
+			diff = check_diff(old_text->str + drop, old_common,
+				       new_text->str, new_common,
+				       &head_len, &tail_len);
+
+			/* Note: to keep offsets correct, we need to
+			 * notify deletions from bottom to top and
+			 * additions from top to bottom
+			 */
+
+			/* Temporarily switch to old text to emit deletion events */
+			priv->snapshot_text = old_text;
+			priv->snapshot_characters = old_characters;
+
+			if (diff)
+				/* Not only did we scroll, but we also modified some bits */
+				/* Delete middle first */
+				emit_text_changed_delete(G_OBJECT(accessible),
+							 old_text->str,
+							 drop + head_len,
+							 old_common - tail_len - head_len);
+
+			/* Delete top */
 			emit_text_changed_delete(G_OBJECT(accessible),
-					priv->snapshot_text->str,
+					old_text->str,
 					0,
-					i);
-			/* Figure out how much text was left, and refresh. */
-			i = strlen(priv->snapshot_text->str + i);
-			inserted = TRUE;
-		}
-		priv->snapshot_contents_invalid = TRUE;
-		vte_terminal_accessible_update_private_data_if_needed(accessible,
-								      NULL,
-								      NULL);
-		/* Any newly-added string data is new, so note that it was
-		 * inserted. */
-		if (inserted) {
-			len = priv->snapshot_text->len;
-			if (len > i) {
-				/* snapshot_text always contains a trailing '\n',
-				 * insertion happens in front of it: bug 657960 */
-				// g_assert(i >= 1);
-                            if (i > 0)
+                                        drop);
+
+			/* Switch to new text */
+			priv->snapshot_text = new_text;
+			priv->snapshot_characters = new_characters;
+
+			g_string_free(old_text, TRUE);
+			g_array_free(old_characters, TRUE);
+
+			if (diff)
+			{
+				/* Add middle */
+				if (tail_len)
+					emit_text_changed_insert(G_OBJECT(accessible),
+								 new_text->str,
+								 head_len,
+								 new_common - head_len - tail_len);
+				else
+					/* Diff was at bottom, merge with addition below */
+					extra_add = new_common - head_len - tail_len;
+			}
+
+			/* Add bottom */
+			if (new_len > new_common - extra_add)
 				emit_text_changed_insert(G_OBJECT(accessible),
 							 priv->snapshot_text->str,
-							 i - 1,
-							 len - i);
-			}
+							 new_common - extra_add,
+							 new_len - (new_common - extra_add));
+		} else {
+			/* Just refresh. */
+			priv->snapshot_contents_invalid = TRUE;
+			vte_terminal_accessible_update_private_data_if_needed(accessible,
+									      NULL,
+									      NULL);
 		}
+
                 vte_terminal_accessible_maybe_emit_text_caret_moved(accessible);
 		return;
 	}
@@ -632,8 +774,9 @@ vte_terminal_accessible_invalidate_cursor(VteTerminal *terminal, gpointer data)
         VteTerminalAccessible *accessible = (VteTerminalAccessible *)data;
 	VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private(accessible);
 
-	_vte_debug_print(VTE_DEBUG_ALLY,
-			"Invalidating accessibility cursor.\n");
+        if (!vte_terminal_get_enable_a11y (terminal))
+                return;
+
 	priv->snapshot_caret_invalid = TRUE;
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
@@ -646,7 +789,11 @@ vte_terminal_accessible_title_changed(VteTerminal *terminal, gpointer data)
 {
         VteTerminalAccessible *accessible = (VteTerminalAccessible *)data;
 
-	atk_object_set_description(ATK_OBJECT(accessible), vte_terminal_get_window_title(terminal));
+        if (!vte_terminal_get_enable_a11y (terminal))
+                return;
+
+        auto const title = vte_terminal_get_window_title(terminal);
+        atk_object_set_description(ATK_OBJECT(accessible), title ? title : "");
 }
 
 /* Reflect visibility-notify events. */
@@ -659,6 +806,9 @@ vte_terminal_accessible_visibility_notify(VteTerminal *terminal,
 	GtkWidget *widget;
 	gboolean visible;
 
+        if (!vte_terminal_get_enable_a11y (terminal))
+                return FALSE;
+
 	visible = event->state != GDK_VISIBILITY_FULLY_OBSCURED;
 	/* The VISIBLE state indicates that this widget is "visible". */
 	atk_object_notify_state_change(ATK_OBJECT(accessible),
@@ -690,6 +840,9 @@ vte_terminal_accessible_selection_changed (VteTerminal *terminal,
 {
         VteTerminalAccessible *accessible = (VteTerminalAccessible *)data;
 
+        if (!vte_terminal_get_enable_a11y (terminal))
+                return;
+
 	g_signal_emit_by_name (accessible, "text_selection_changed");
 }
 
@@ -739,8 +892,6 @@ _vte_terminal_accessible_init (VteTerminalAccessible *accessible)
 {
         VteTerminalAccessiblePrivate *priv = (VteTerminalAccessiblePrivate *)_vte_terminal_accessible_get_instance_private (accessible);
 
-	_vte_debug_print(VTE_DEBUG_ALLY, "Initialising accessible peer.\n");
-
 	priv->snapshot_text = NULL;
 	priv->snapshot_characters = NULL;
 	priv->snapshot_linebreaks = NULL;
@@ -760,8 +911,6 @@ vte_terminal_accessible_finalize(GObject *object)
         GtkWidget *widget;
 	gint i;
 
-	_vte_debug_print(VTE_DEBUG_ALLY, "Finalizing accessible peer.\n");
-
         widget = gtk_accessible_get_widget (GTK_ACCESSIBLE(accessible));
 
 	if (widget != NULL) {
@@ -831,11 +980,6 @@ vte_terminal_accessible_get_text(AtkText *text,
 	vte_terminal_accessible_update_private_data_if_needed(accessible,
 							      NULL, NULL);
 
-	_vte_debug_print(VTE_DEBUG_ALLY,
-			"Getting text from %d to %d of %d.\n",
-			start_offset, end_offset,
-			priv->snapshot_characters->len);
-
 	/* If the requested area is after all of the text, just return an
 	 * empty string. */
 	if (start_offset >= (int) priv->snapshot_characters->len) {
@@ -883,18 +1027,6 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 
         auto impl = IMPL_FROM_ACCESSIBLE(text);
 
-	_vte_debug_print(VTE_DEBUG_ALLY,
-			"Getting %s %s at %d of %" G_GSIZE_FORMAT " .\n",
-			(direction == direction_current) ? "this" :
-			((direction == direction_next) ? "next" : "previous"),
-			(boundary_type == ATK_TEXT_BOUNDARY_CHAR) ? "char" :
-			((boundary_type == ATK_TEXT_BOUNDARY_LINE_START) ? "line (start)" :
-			((boundary_type == ATK_TEXT_BOUNDARY_LINE_END) ? "line (end)" :
-			((boundary_type == ATK_TEXT_BOUNDARY_WORD_START) ? "word (start)" :
-			((boundary_type == ATK_TEXT_BOUNDARY_WORD_END) ? "word (end)" :
-			((boundary_type == ATK_TEXT_BOUNDARY_SENTENCE_START) ? "sentence (start)" :
-			((boundary_type == ATK_TEXT_BOUNDARY_SENTENCE_END) ? "sentence (end)" : "unknown")))))),
-			offset, vte_char_attr_list_get_size(&priv->snapshot_attributes));
 	g_assert(priv->snapshot_text != NULL);
 	g_assert(priv->snapshot_characters != NULL);
 	if (offset >= (int) priv->snapshot_characters->len) {
@@ -1086,9 +1218,6 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 					break;
 				}
 			}
-			_vte_debug_print(VTE_DEBUG_ALLY,
-					"Character %d is on line %d.\n",
-					offset, line);
 			/* Perturb the line number to handle before/at/after. */
 			line += direction;
 			line = MIN(line, priv->snapshot_linebreaks->len - 1);
@@ -1099,9 +1228,6 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 			line = MIN(line, priv->snapshot_linebreaks->len - 1);
 			end = g_array_index(priv->snapshot_linebreaks,
 						    int, line);
-			_vte_debug_print(VTE_DEBUG_ALLY,
-					"Line runs from %d to %d.\n",
-					start, end);
 			break;
 		case ATK_TEXT_BOUNDARY_SENTENCE_START:
 		case ATK_TEXT_BOUNDARY_SENTENCE_END:
